#include "CivOptSchedEngine.h"
#include "CivInpConvertor.h"
#include "GenAlg.h"
#include "FirstOptScheduling.h"
#include "CivPumpHelper.h"
#include "Genome.h"
#include "CivCommonUtils.h"
#include "CivSysLog.h"
#include "CivSchedulingCompute.h"

CivOptSchedEngine::CivOptSchedEngine(const string& uri)
	:mUri(uri),mMutRate(0.45),mCrossRate(0.07), popsize(10)
{
	// ʼ

}

bool CivOptSchedEngine::optimalScheduling(int time)
{
	// ʼ
	if (!beginCompute())
	{
		return false;
	}
		
	// ʼŴ㷨
	std::shared_ptr<OptScheduling> optScheduling(new FirstOptScheduling());

	// ת
	auto minIter = mMinMonitorVals.begin();
	map<string, double> minMonitorVals;
	for (; minIter != mMinMonitorVals.end(); minIter++)
	{
		string code = minIter->first;
		auto szNoIter = find(mMonitorsVec.begin(), mMonitorsVec.end(), code);
		if (szNoIter != mMonitorsVec.end())
		{
			code = *szNoIter;
		}
		minMonitorVals.insert(pair<string, double>(code, minIter->second));
	}

	optScheduling->setMinMonitorsVals(minMonitorVals);

	// ת
	auto maxIter = mMaxMonitorVals.begin();
	map<string, double> maxMonitorVals;
	for (; maxIter != mMaxMonitorVals.end(); maxIter++)
	{
		string code = maxIter->first;
		auto szNoIter = find(mMonitorsVec.begin(), mMonitorsVec.end(), code);
		if (szNoIter != mMonitorsVec.end())
		{
			code = *szNoIter;
		}
		maxMonitorVals.insert(pair<string, double>(code, maxIter->second));
	}
	optScheduling->setMaxMonitorVals(mMaxMonitorVals);

	GenAlg genAlg(optScheduling);
	genAlg.setPumpVec(mPumpVec);

	// Ⱥʼ
	popsize = 10;
	genAlg.init(popsize, mMutRate, mCrossRate, mGenLength, 0, 1);
	
	// ʼ
	int index = 0;

	// 
	int generation = 20;

	mSchedulingCompute->clearLastVals();
	while (index++ < generation)
	{
		string infoLog = "" + to_string(index) + "ѵʼ";
		CivSysLog::getInstance()->info(infoLog, "CivOptSchedEngine", __FUNCTION__);

		// ƽ
		if (index >1)
		{
			double gg = 0.9;
		}
		vector<map<string, float>>  pumpSpeedMapVec;
		genAlg.decoding(pumpSpeedMapVec);

		// Ⱥÿ
		size_t mSize = pumpSpeedMapVec.size();
		for (int i = 0; i < mSize; i++)
		{
			map<string, float> pumpSpeedMap = pumpSpeedMapVec[i];
			mSchedulingCompute->updatePumpSpeed(pumpSpeedMap);
			if (!mSchedulingCompute->calculate())
				continue;

			map<string, float>::const_iterator const_iter = pumpSpeedMap.cbegin();

			string logStr = "" + to_string(i) + "壺";
			for (; const_iter != pumpSpeedMap.cend(); const_iter++)
			{
				logStr.append(const_iter->first + ":" + to_string(const_iter->second)+"   ");
			}

			// ȡļֵ
			map<string, double> monitorCalcMapValues;
			mSchedulingCompute->getMonitorsValue(mPeriod, monitorCalcMapValues);

			// ȡĵֵ
			float cost = mSchedulingCompute->totalEnergy(mPeriod);
			
			// ʶ
			double fitness = genAlg.fitnessfunction(cost, monitorCalcMapValues);

			if (fitness >= 1)
				fitness = 0;

			logStr.append(" Ӧֵ:" + to_string(fitness));
			CivSysLog::getInstance()->info(logStr, "CivOptSchedEngine", __FUNCTION__);

			// ¸Ӧ
			genAlg.updateGenomeFitNess(i, fitness);
			
			mSchedulingCompute->clearLastVals();
		}

		// һȺ
		std::vector<Genome> newGenmeVec;
		genAlg.epoch(newGenmeVec);
	}

	genAlg.calculateBestWorstAvTot();
	Genome genome = genAlg.bestFitness();
	vector<double> genVec = genome.vecGenome;

	// 
	size_t totalGenVec = genVec.size();
	size_t pumpSize = mPumpVec.size();
	if (totalGenVec != pumpSize)
	{
		CivSysLog::getInstance()->error("ˮøֵƥ", "CivOptSchedEngine", __FUNCTION__);
		return false;
	}
	
	// ˮã
	string infoStr  = "ˮֵ";
	for (int i = 0; i < totalGenVec; i++)
	{
		infoStr.append(mPumpVec[i] +": " + to_string(genVec[i]) +"  ");
		mPumpBestSpeed.insert(pair<string, double>(mPumpVec[i], genVec[i]));
	}
	infoStr.append(" Ӧȣ" + to_string(genome.fitness));
	CivSysLog::getInstance()->error(infoStr, "CivOptSchedEngine", __FUNCTION__);

	mSchedulingCompute->clearLastVals();
	// һˮ㣬Ⱥֵ
	mSchedulingCompute->updatePumpSpeed(mPumpBestSpeed);
	if (!mSchedulingCompute->calculate())
		return false;

	// ŵֵ
	mSchedulingCompute->getMonitorsValue(mPeriod,mAfterSchedulingResults);

	// 
	endCompute();

	return true;
}

void CivOptSchedEngine::serializeToInpFile(string& inpName)
{

}

void CivOptSchedEngine::schedulingResultToJson(string& json)
{
	// ˮ:
	json.append("{\"ˮ״̬\":{");

	map<string, float>::iterator iter = mPumpBestSpeed.begin();
	for (;iter!= mPumpBestSpeed.end();iter++)
	{
		// ˮñ
		string szNo = iter->first;

		// ת
		auto pumpIter = mPumpSzMap.find(szNo);
		if (pumpIter != mPumpSzMap.end())
		{
			szNo = pumpIter->second;
		}

		json.append(szNo + ":" + to_string(iter->second));
		json.append(",");
	}
	json = json.substr(0, json.length() - 1);
	json.append("},");

	// 
	json.append("\":\"");

	json.append("{");
	for (auto iter = mBforeSchedulingResults.begin(); iter != mBforeSchedulingResults.end(); iter++)
	{
		string monitorName = iter->first;

		double beforeMonitorVal = iter->second;

		auto afteriter = mAfterSchedulingResults.find(monitorName);
		if (afteriter == mAfterSchedulingResults.end())
			continue;

		double afterMonitorvVal = afteriter->second;

		// 滻code
		auto monitorIter = mMonitorsSnMap.find(monitorName);
		if (monitorIter != mMonitorsSnMap.end())
		{
			monitorName = monitorIter->second;
		}

		json.append("\""+monitorName+"\":[");
		json.append(to_string(beforeMonitorVal));
		json.append(",");
		json.append(to_string(afterMonitorvVal));
		json.append("],");
	}
	json = json.substr(0, json.length() - 1);
	json.append("}");

}

void CivOptSchedEngine::updateMonitorsVals(const map<string, double>& monitorVals)
{
	mCurrentMonitorVals = monitorVals;
}

bool CivOptSchedEngine::beginCompute()
{
	// תinpļ
	CivInpConvertor convertor(mUri);
	string inpFile = convertor.convertBaseInp();

	mSchedulingCompute = std::make_shared<CivSchedulingCompute>(CivSchedulingCompute::PumpScheduling);
	

	// ȡˮø,ˮøȷ볤
	CivPumpHelper pumpHelper(mUri);
	mGenLength = pumpHelper.getPumpNumbers();
	if (mGenLength <= 0)
	{
		CivSysLog::getInstance()->error("CivOptSchedEngine","ȡˮ",__FUNCTION__);
		return false;
	}
		
	// ȡźcodeӳ
	map<string, string> snToCodeMap;
	pumpHelper.getMap("", "code", snToCodeMap);

	// ȡż
	pumpHelper.getPumpSn(mPumpVec, mPumpSzMap);
	pumpHelper.getMonitors(mMonitorsVec,mMonitorsSnMap);

	/*string temp = "{";
	for (int i = 0; i < mMonitorsVec.size(); i++)
	{
		temp.append("\\\"" + mMonitorsVec[i] + "\\\":{\\\"min\\\":20,\\\"max\\\":48},");
	}
	temp = temp.substr(0, temp.length() - 1);
	temp.append("}");*/

	// ˮ
	mSchedulingCompute->setMonitors(mMonitorsVec);
	if (!mSchedulingCompute->openFile(inpFile))
	{
		CivSysLog::getInstance()->info(inpFile,"CivOptSchedEngine",__FUNCTION__);
		return false;
	}
	
	// ԭʼһˮǰˮ
	if (!mSchedulingCompute->calculate())
		return false;

	// ԭʼģֵ
	mSchedulingCompute->getMonitorsValue(mPeriod, mBforeSchedulingResults);
}

void CivOptSchedEngine::endCompute()
{
	mSchedulingCompute->close();
}

void CivOptSchedEngine::setMonitors(const vector<string>& monitor)
{
	mMonitorsVec = monitor;
}
